#include "Matrix.h"

double Matrix::get(size_t const& i,size_t const& j) const{return comp[i][j];}
void Matrix::set(size_t const& i, size_t const& j,double const& nb){comp[i][j]=nb;invUpToDate=false;}
/*void Matrix::setDim(size_t const& i, size_t const& j){
	//!COMPLETER
}*/
size_t Matrix::getDimi() const{return comp.size();}
size_t Matrix::getDimj() const{if(getDimi()==0)return 0;return comp[0].size();}

Matrix Matrix::operator*(double const& nb)const{
	Matrix m(*this);
	for(size_t i(0);i<(*this).getDimi();i++){
		for(size_t j(0);j<(*this).getDimj();j++){
			m.set(i,j,get(i,j)*nb);
		}
	}
	return m;
}
Matrix& Matrix::operator*=(double const& nb){invUpToDate=false;return (*this)= ( (*this)*nb );}
Matrix operator*(double const& nb, Matrix const& m){return m*nb;}
Matrix& operator*=(double const& nb, Matrix& m){m.invUpToDate=false;return m*=nb;}

Matrix Matrix::getInv() {if(!invUpToDate)updateInv();return Matrix(inv);}
bool Matrix::updateInv(){
	double det = (*this).det();
	if(det==0) {cerr <<"Matrix: det=0, not invertible"<<endl;return false;}
	if(getDimi()==1){return get(0,0)/det;}
	if(getDimi()==2){
		Matrix result(2);
		result.set(0,0,get(1,1)/det);
		result.set(0,1,-get(1,0)/det);
		result.set(1,0,-get(0,1)/det);
		result.set(1,1,get(0,0)/det);
		inv=result.comp;
		return true;
	}
	if(getDimj()==3){
		Matrix result(3);
		result.set(0,0,( get(1,1)*get(2,2) - get(1,2)*get(2,1) )/det);
		result.set(0,1,( get(0,2)*get(2,1) - get(0,1)*get(2,2) )/det);
		result.set(0,2,( get(0,1)*get(1,2) - get(0,2)*get(1,1) )/det);

		result.set(1,0,( get(1,2)*get(2,0) - get(1,0)*get(2,2) )/det);
		result.set(1,1,( get(0,0)*get(2,2) - get(0,2)*get(2,0) )/det);
		result.set(1,2,( get(0,2)*get(1,0) - get(0,0)*get(1,2) )/det);

		result.set(2,0,( get(1,0)*get(2,1) - get(1,1)*get(2,0) )/det);
		result.set(2,1,( get(0,1)*get(2,0) - get(0,0)*get(2,1) )/det);
		result.set(2,2,( get(0,0)*get(1,1) - get(0,1)*get(1,0) )/det);
		inv=result.comp;
		return true;
	}
	Matrix result(getDimi());
	for(size_t i=0;i<getDimi();i++){
		for(size_t j=0;j<getDimi();j++){
			result.set(i,j,min(i,j).det());
		}
	}
	inv=result.comp;
	invUpToDate=true;
	return true;


	//!Completer
}
double Matrix::det(size_t const& j) const{
	if(getDimi()!=getDimj()) {cerr<<"Matrix: det: NOT SQUARE"<<endl;return 0;}//PAS CARRE !
	if(getDimi()==1) return get(0,0);
	if(getDimi()==2) return get(0,0)*get(1,1)-get(0,1)*get(1,0);
	if(getDimi()==3) return get(0,0)*get(1,1)*get(2,2) + get(0,1)*get(1,2)*get(2,0) + get(0,2)*get(1,0)*get(2,1)
						   -get(0,0)*get(1,2)*get(2,1) - get(0,1)*get(1,0)*get(2,2) - get(0,2)*get(1,1)*get(2,0);
	//dev ligne :
	double result=0;
	for(size_t i=0;i<getDimi();i++){
		result += min(i,0).det(j+1) * (1-2*((i+j)%2)); //-1^n
	}
	return result;
}

Matrix Matrix::min(size_t const& i, size_t const& j)const{
	Matrix result(getDimi()-1,getDimj()-1);
	for(size_t iterI=0;iterI<getDimi()-1;iterI++){
		for(size_t iterJ=0;iterJ<getDimj()-1;iterJ++){
			if(iterI<i){
				if(iterJ<j) result.set(iterI,iterJ,get(iterI,iterJ));
				else result.set(iterI,iterJ,get(iterI,iterJ+1));
			}
			else {
				if(iterJ<j) result.set(iterI,iterJ,get(iterI+1,iterJ));
				else result.set(iterI,iterJ,get(iterI+1,iterJ+1));
			}
		}
	}
	return result;
}

Matrix Matrix::operator*(Matrix const& m) const{
	//si le calc impossible, retourne une matrice vide
	//!on peut modifier pour avoir des cas de dim different avec setDim
	if((*this).getDimj()!=m.getDimi()) {return Matrix();}
	//---
	Matrix result((*this).getDimi(),m.getDimj());
	for(size_t i=0;i<result.getDimi();i++){
		for(size_t j=0;j<result.getDimj();j++){
			for(size_t k=0;k<(*this).getDimi();k++){
				result.set(i,j,result.get(i,j)+(*this).get(i,k)*(*this).get(k,j));
			}
		}
	}
	return result;
}
Matrix& Matrix::operator*=(Matrix const& m){invUpToDate=false;return (*this)=(*this)*m;}

Vect Matrix::operator*(Vect const& v) const{
	if(v.getDim()!=getDimj())return Vect();
	Vect result(getDimi());
	for(size_t i=0;i<getDimi();i++){
		for(size_t k=0;k<getDimj();k++){
			result.set(i,result.get(i)+get(i,k)*v.get(k));
		}
	}
	return result;
}
Matrix operator*(Vect const& v,Matrix const& m){
	if(m.getDimi()!=1) return Matrix();
	Matrix result(v.getDim(),m.getDimj());
	for(size_t i=0;i<result.getDimi();i++){
		for(size_t j=0;j<result.getDimj();j++){
			result.set(i,j, v.get(i)*m.get(0,j));
		}
	}
	return result;
}

Matrix Matrix::operator+(Matrix const& m) const{
	//si les dim ne collent pas
	//! on peut modifier plus tard pour add des Matrix de taille diffrente
	if((*this).getDimi()!=m.getDimi() || (*this).getDimj()!=m.getDimj()){cerr<<"Matrix: operator+ NOT COMPATIBLE"<<endl;return Matrix(0);}
	//---
	Matrix result(getDimi(),getDimj());
	for(size_t i=0;i<getDimi();i++){
		for(size_t j=0;j<getDimj();j++){
			result.set(i,j,get(i,j)+m.get(i,j));
		}
	}
	return result;
}
Matrix& Matrix::operator+=(Matrix const&m){invUpToDate=false;return (*this)=(*this)+m;}
Matrix Matrix::operator-(Matrix const& m) const{return (*this)+(-m);}
Matrix& Matrix::operator-=(Matrix const& m){invUpToDate=false;return (*this)=(*this)-m;}
Matrix Matrix::operator-() const{
	Matrix result(getDimi(),getDimj());
	for(size_t i=0;i<getDimi();i++){
		for(size_t j=0;j<getDimj();j++){
			result.set(i,j,-(*this).get(i,j));
		}
	}
	return result;
}

bool Matrix::operator==(Matrix const& m)const{
	if(getDimi()!=m.getDimi()|| getDimj()!=m.getDimj())return false;

	for(size_t i=0;i<getDimi();i++){
		for(size_t j=0;j<getDimj();j++){
			if(get(i,j)!=m.get(i,j))return false;
		}
	}
	return true;
}

ostream& Matrix::disp(ostream& out) const{
	out<<"---MATRIX---"<<endl;
	for(size_t i=0;i<getDimi();i++){
		out<<"|";
		for(size_t j=0;j<getDimj();j++){
			out<<get(i,j)<<" ";
		}
		out<<endl;
	}
	out<<"-------------"<<endl;
	return out;
}
ostream& operator<<(ostream& out, Matrix const& m){
	return m.disp(out);
}

Matrix Matrix::transpose() const{
	Matrix result(getDimj(),getDimi());
	for(size_t i=0;i<getDimi();i++){
		for(size_t j=0;j<getDimj();j++){
			result.set(j,i,get(i,j));
		}
	}
	return result;
}

vector<vector<double > > Matrix::convertTab(){return comp;}


Matrix rotationMatrix(double theta,double x,double y,double z){
    Matrix result(3,1);//matrix id

    double c=cos(theta);
    double s=sin(theta);

    result.set(0,0, x*x + (1-x*x)*c );
    result.set(0,1, x*y*(1-c) - z*s);
    result.set(0,2, x*z*(1-c) + y*s);

    result.set(1,0, x*y*(1-c) + z*s);
    result.set(1,1, y*y + (1-y*y)*c);
    result.set(1,2, y*z*(1-c) - x*s);

    result.set(2,0, x*z*(1-c) - y*s);
    result.set(2,1, y*z*(1-c) + x*s);
    result.set(2,2, z*z + (1-z*z)*c);
	
    return result;
}


//CONSTR---
Matrix::Matrix(size_t const& i,size_t const& j, double const& nb):comp(i,vector<double>(j,nb)){}
Matrix::Matrix(size_t const& i):comp(i,vector<double>(i,0)){}
Matrix::Matrix(vector<vector<double> >const& m):comp(m){}
//Matrix::Matrix(initializer_list<initializer_list<double> > const& m):comp(m){}
Matrix::Matrix(size_t const& i, double const& nb):comp(i,vector<double>(i,0)){
	for(size_t it=0;it<i;it++){
		set(it,it,nb);
	}
}






